package ru.vendetti.bitcoin_summarizer

import android.annotation.SuppressLint
import android.content.Context
import android.content.pm.PackageManager
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.wrapContentSize
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Menu
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.ButtonColors
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Card
import androidx.compose.material3.CenterAlignedTopAppBar
import androidx.compose.material3.DrawerValue
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.ModalNavigationDrawer
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.material3.rememberDrawerState
import androidx.compose.material3.rememberTopAppBarState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.platform.ComposeView
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.compose.ui.window.Dialog
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.google.gson.Gson
import com.patrykandpatrick.vico.compose.cartesian.CartesianChartHost
import com.patrykandpatrick.vico.compose.cartesian.axis.rememberAxisLabelComponent
import com.patrykandpatrick.vico.compose.cartesian.axis.rememberAxisLineComponent
import com.patrykandpatrick.vico.compose.cartesian.axis.rememberBottom
import com.patrykandpatrick.vico.compose.cartesian.axis.rememberStart
import com.patrykandpatrick.vico.compose.cartesian.layer.rememberLine
import com.patrykandpatrick.vico.compose.cartesian.layer.rememberLineCartesianLayer
import com.patrykandpatrick.vico.compose.cartesian.rememberCartesianChart
import com.patrykandpatrick.vico.compose.cartesian.rememberVicoScrollState
import com.patrykandpatrick.vico.compose.cartesian.rememberVicoZoomState
import com.patrykandpatrick.vico.compose.common.component.rememberTextComponent
import com.patrykandpatrick.vico.compose.common.component.shapeComponent
import com.patrykandpatrick.vico.compose.common.fill
import com.patrykandpatrick.vico.compose.common.shape.rounded
import com.patrykandpatrick.vico.compose.common.vicoTheme
import com.patrykandpatrick.vico.core.cartesian.Zoom
import com.patrykandpatrick.vico.core.cartesian.axis.HorizontalAxis
import com.patrykandpatrick.vico.core.cartesian.axis.VerticalAxis
import com.patrykandpatrick.vico.core.cartesian.data.CartesianChartModelProducer
import com.patrykandpatrick.vico.core.cartesian.data.CartesianLayerRangeProvider
import com.patrykandpatrick.vico.core.cartesian.data.lineSeries
import com.patrykandpatrick.vico.core.cartesian.decoration.HorizontalLine
import com.patrykandpatrick.vico.core.cartesian.layer.LineCartesianLayer
import com.patrykandpatrick.vico.core.common.Position
import com.patrykandpatrick.vico.core.common.component.LineComponent
import com.patrykandpatrick.vico.core.common.component.TextComponent
import com.patrykandpatrick.vico.core.common.shape.CorneredShape
import kotlinx.coroutines.launch
import ru.vendetti.bitcoin_summarizer.ui.theme.BitcoinSummarizerTheme
import ru.vendetti.bitcoin_summarizer.ui.theme.Flame
import ru.vendetti.bitcoin_summarizer.ui.theme.Green2
import java.io.FileInputStream
import java.io.FileOutputStream
import java.lang.StringBuilder
import java.text.DecimalFormat
import java.time.Instant
import java.time.format.DateTimeFormatter

class CryptoFragment: Fragment() {
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        return inflater.inflate(R.layout.fragment_compose_view,
            container,
            false).apply {
            findViewById<ComposeView>(R.id.compose_view).setContent {
                BitcoinSummarizerTheme {
                    CryptoComposable()
                }
            }
        }
    }
}

@Composable
fun ShowAlertDialog(
    onDismissRequest: () -> Unit,
    onConfirmation: () -> Unit,
    dialogTitle: String,
    dialogText: String,
) {
    AlertDialog(
        icon = {},
        title = {
            Text(text = dialogTitle)
        },
        text = {
            Text(text = dialogText)
        },
        onDismissRequest = {
            onDismissRequest()
        },
        confirmButton = {},
        dismissButton = {
            TextButton(
                onClick = {
                    onDismissRequest()
                }
            ) {
                Text(
                    text = "Окей",
                    color = MaterialTheme.colorScheme.onPrimary
                )
            }
        },
        containerColor = MaterialTheme.colorScheme.primary,
        titleContentColor = MaterialTheme.colorScheme.onPrimary,
        textContentColor = MaterialTheme.colorScheme.onPrimary,
    )
}

@SuppressLint("MutableCollectionMutableState", "SimpleDateFormat")
@OptIn(ExperimentalMaterial3Api::class)
@Preview
@Composable
fun CryptoComposable() {
    val openAlertDialog1 = remember { mutableStateOf(false) }
    val openAlertDialog2 = remember { mutableStateOf(false) }

    // Создаем репозиторий для работы с API
    val cryptoRepository = remember { CryptoRepository(RetrofitClient.apiService) }
    // Состояния для хранения результатов запросов
    var bitcoinTicker by remember { mutableStateOf(TickerData()) }
    var globalData by remember { mutableStateOf(GlobalResponse()) }
    var fearGreedDataList by remember { mutableStateOf(ArrayList<FearAndGreedData>()) }

    var fearGreedIndexDaysCount by remember { mutableIntStateOf(30) }

    val gson = Gson()

    val context = LocalContext.current

    when {
        openAlertDialog1.value -> {
            ShowAlertDialog(
                onDismissRequest = {openAlertDialog1.value = false},
                onConfirmation = {},
                dialogTitle = "Внимание!",
                dialogText = "Отсутствует интернет-подключение, загружаю сохраненные данные. " +
                        "" +
                        "Обратите внимание, что данные могут быть неактуальными.",
            )
        }
        openAlertDialog2.value -> {
            ShowAlertDialog(
                onDismissRequest = {
                    openAlertDialog2.value = false
                    App.INSTANCE.router.exit()
                                   },
                onConfirmation = {},
                dialogTitle = "Внимание!",
                dialogText = "При первом подключении необходимо подключение к интернету!",
            )
        }
    }

    // Запускаем корутину для выполнения сетевых запросов
    LaunchedEffect(fearGreedIndexDaysCount, context) {
        try {
            // Запрос Bitcoin Ticker
            val tickerResponse = cryptoRepository.fetchBitcoinTicker()
            bitcoinTicker = tickerResponse!!.first()

            // Запрос глобальных данных
            val globalResponse = cryptoRepository.fetchGlobalData()
            globalData = globalResponse!!

            // Запрос индекса страха и жадности
            val fearResponse = cryptoRepository.fetchFearAndGreedData(fearGreedIndexDaysCount)
            fearGreedDataList = fearResponse?.dataList as ArrayList<FearAndGreedData>
            fearGreedDataList.reverse()

            // *** Сохраняем на диск ***
            // 1) Сбор данных в один класс
            val fullStatistics = StatisticsFull(
                FGI_list = fearGreedDataList,
                TickerData = bitcoinTicker,
                GlobalData = globalData,
            )

            // 2) Перевод класса в json-строку
            val statisticsJsonString = gson.toJson(fullStatistics)

            // 3) Сохранение в файл во внутреннем хранилище
            val fos: FileOutputStream =
                context.openFileOutput("statistics.txt", Context.MODE_PRIVATE)
            fos.write(statisticsJsonString.toByteArray())
            fos.flush()
            fos.close()
        }
        catch (e: Exception) {
            try {
                // *** Загрузка данных из файла ***
                // 1) Получение строки из файла
                val fis: FileInputStream =
                    context.openFileInput("statistics.txt")
                var a: Int
                val temp = StringBuilder()
                while (fis.read().also { a = it } != -1) {
                    temp.append(a.toChar())
                }

                // Показ предупреждения.
                openAlertDialog1.value = true

                val statisticsJsonString = temp.toString()

                // 2) Перевод строки в класс
                val statisticsFull = gson.fromJson(statisticsJsonString, StatisticsFull::class.java)

                // 3) Распределение данных по переменным
                bitcoinTicker = statisticsFull.TickerData
                globalData = statisticsFull.GlobalData
                fearGreedDataList = statisticsFull.FGI_list
            }
            catch (e: Exception) {
                openAlertDialog2.value = true
            }
        }
    }

    val modelProducer = remember { CartesianChartModelProducer() }

    LaunchedEffect(fearGreedDataList) {
        modelProducer.runTransaction {
            var numberValues = Array(fearGreedDataList.count()) {
                    index ->
                fearGreedDataList[index]
                    .value.toInt()
            }

            if(numberValues.isEmpty())
                numberValues = Array(1) {0}

            lineSeries { series(numberValues.toList()) }
        }
    }

    val scrollBehavior = TopAppBarDefaults.enterAlwaysScrollBehavior(rememberTopAppBarState())

    // Отображаем результаты на странице

    val drawerState = rememberDrawerState(initialValue = DrawerValue.Closed)
    val scope = rememberCoroutineScope()

    ModalNavigationDrawer(
        drawerState = drawerState,
        drawerContent = {
            MyHamburgerModal(ScreenKey.Crypto)
        }
    ) {
        Scaffold(
            modifier = Modifier
                .nestedScroll(scrollBehavior.nestedScrollConnection),

            topBar = {
                CenterAlignedTopAppBar(
                    colors = TopAppBarDefaults.topAppBarColors(
                        containerColor = MaterialTheme.colorScheme.primary,
                        scrolledContainerColor = MaterialTheme.colorScheme.primary,
                        titleContentColor = MaterialTheme.colorScheme.onPrimary,
                        navigationIconContentColor = MaterialTheme.colorScheme.onPrimary,
                        actionIconContentColor = MaterialTheme.colorScheme.onPrimary,
                    ),
                    title = {
                        Text(
                            "Crypto statistics",
                            maxLines = 1,
                            overflow = TextOverflow.Ellipsis
                        )
                    },
                    navigationIcon = {
                        IconButton(onClick = {
                            scope.launch {
                                drawerState.apply {
                                    if(isClosed) open() else close()
                                }
                            }
                        }) {
                            Icon(
                                imageVector = Icons.Filled.Menu,
                                contentDescription = "Navigation hamburger menu"
                            )
                        }
                    },
                    scrollBehavior = scrollBehavior
                )
            },
        )
        { innerPadding ->
            Box(
                modifier = Modifier
                    .background(Color.Transparent)
                    .padding(innerPadding)
                    .verticalScroll(rememberScrollState())
            ) {
                Column(
                    modifier = Modifier
                        .padding(16.dp, 16.dp, 16.dp, 36.dp)
                ) {
                    /* Fear Greed Chart Start */
                    Spacer(modifier = Modifier.height(16.dp))
                    Text(
                        "Индекс страха/жадности",
                        modifier = Modifier
                            .align(alignment = Alignment.CenterHorizontally),
                        fontSize = 24.sp
                    )
                    Spacer(modifier = Modifier.height(16.dp))
                    if (fearGreedDataList.count() > 1) {
                        CartesianChartHost(
                            zoomState = rememberVicoZoomState(
                                zoomEnabled = true,
                                initialZoom = Zoom.x(fearGreedIndexDaysCount.toDouble()),
                            ),
                            scrollState = rememberVicoScrollState(scrollEnabled = true),
                            chart = rememberCartesianChart(
                                rememberLineCartesianLayer(
                                    lineProvider = LineCartesianLayer.LineProvider.series(
                                        vicoTheme.lineCartesianLayerColors.map { _ ->
                                            LineCartesianLayer
                                                .rememberLine(
                                                    LineCartesianLayer.LineFill.single(
                                                        fill(MaterialTheme.colorScheme.onPrimary)
                                                    )
                                                )
                                        }
                                    ),
                                    rangeProvider = remember {
                                        CartesianLayerRangeProvider.fixed(
                                            minY = 0.0,
                                            maxY = 100.0,
                                        )
                                    }
                                ),
                                startAxis = VerticalAxis.rememberStart(
                                    title = "FGI",
                                    titleComponent = rememberTextComponent(MaterialTheme.colorScheme.onPrimary),
                                    line = rememberAxisLineComponent(fill(MaterialTheme.colorScheme.onPrimary)),
                                    label = rememberAxisLabelComponent(MaterialTheme.colorScheme.onPrimary)
                                ),
                                bottomAxis = HorizontalAxis.rememberBottom(
                                    valueFormatter = { _, value, _ ->
                                        val date = DateTimeFormatter.ISO_INSTANT
                                            .format(Instant.ofEpochSecond(fearGreedDataList[value.toInt()].timestamp.toLong()))
                                        val dateComponents = date.split("T")[0].split("-")
                                        // returns dd.mm
                                        "${dateComponents[2]}.${dateComponents[1]}"
                                    },
                                    title = "last $fearGreedIndexDaysCount days",
                                    titleComponent = rememberTextComponent(MaterialTheme.colorScheme.onPrimary),
                                    line = rememberAxisLineComponent(fill(MaterialTheme.colorScheme.onPrimary)),
                                    label = rememberAxisLabelComponent(MaterialTheme.colorScheme.onPrimary)
                                ),
                                decorations = listOf(
                                    remember {
                                        HorizontalLine(
                                            y = { 25.toDouble() },
                                            line = LineComponent(fill(Flame), 1f),
                                            labelComponent = TextComponent(
                                                background =
                                                    shapeComponent(
                                                        fill(Flame),
                                                        CorneredShape.rounded(
                                                            topLeft = 4.dp,
                                                            topRight = 4.dp
                                                        )
                                                    ),
                                            ),
                                            label = { "Страх" },
                                            verticalLabelPosition = Position.Vertical.Top
                                        )
                                    },
                                    remember {
                                        HorizontalLine(
                                            y = { 70.toDouble() },
                                            line = LineComponent(fill(Green2), 1f),
                                            labelComponent = TextComponent(
                                                background =
                                                    shapeComponent(
                                                        fill(Green2),
                                                        CorneredShape.rounded(
                                                            bottomLeft = 4.dp,
                                                            bottomRight = 4.dp
                                                        )
                                                    ),
                                            ),
                                            label = { "Жадность" },
                                            verticalLabelPosition = Position.Vertical.Bottom
                                        )
                                    }
                                ),
                            ),
                            modelProducer = modelProducer,
                        )
                    } else {
                        Text("Загрузка...")
                    }
                    /* Fear Greed Chart End */
                    Spacer(modifier = Modifier.height(16.dp))
                    HorizontalDivider(thickness = 2.dp)
                    Spacer(modifier = Modifier.height(16.dp))
                    Text(
                        "Данные о Биткоине",
                        modifier = Modifier
                            .align(alignment = Alignment.CenterHorizontally),
                        fontSize = 24.sp
                    )
                    Spacer(modifier = Modifier.height(16.dp))
                    val decimalFormatter = DecimalFormat("0.00")
                    val largeNumberFormatter = DecimalFormat("#,###")

                    if (bitcoinTicker.lastUpdated.isNotEmpty()) {
                        Text("Текущая цена: \n \$ ${decimalFormatter.format(bitcoinTicker.priceUsd.toFloat())}\n")
                        Text("Суточный оборот: \n \$ ${largeNumberFormatter.format(bitcoinTicker.volume24hUsd.toFloat())}\n")
                        Text("Капитализация: \n \$ ${largeNumberFormatter.format(bitcoinTicker.marketCapUsd.toFloat())}\n")
                        Text(
                            "Изменение курса за: " +
                                    "\n Сутки: ${decimalFormatter.format(bitcoinTicker.percentChange24h.toFloat())}% " +
                                    "\n Неделю: ${decimalFormatter.format(bitcoinTicker.percentChange7d.toFloat())}%\n"
                        )

                        val humanDate = DateTimeFormatter.ISO_INSTANT
                            .format(Instant.ofEpochSecond(bitcoinTicker.lastUpdated.toLong()))

                        Text(
                            "Время последнего обновления: \n ${humanDate.split("T")[0]}, " +
                                    "${
                                        humanDate.split("T")[1].replace(
                                            Regex(":[0-9]+[A-Z]"),
                                            ""
                                        )
                                    }\n"
                        )
                    } else {
                        Text("Загрузка...")
                    }
                    HorizontalDivider(thickness = 2.dp)
                    Spacer(modifier = Modifier.height(16.dp))
                    Text(
                        "Глобальные данные",
                        modifier = Modifier
                            .align(alignment = Alignment.CenterHorizontally),
                        fontSize = 24.sp
                    )
                    Spacer(modifier = Modifier.height(16.dp))
                    if (globalData.totalMarketCapUsd.isNotEmpty()) {
                        Text(
                            "Общая капитализация крипторынка: \n \$ ${
                                largeNumberFormatter.format(
                                    globalData.totalMarketCapUsd.toFloat()
                                )
                            }\n"
                        )
                        Text("Всего тикеров: \n ${globalData.activeCryptocurrencies}\n")
                        Text(
                            "Суточный оборот всех криптовалют: \n \$ ${
                                largeNumberFormatter.format(
                                    globalData.total24hVolumeUsd.toFloat()
                                )
                            }\n"
                        )
                        Text("Процент доминации Биткоина: \n ${decimalFormatter.format(globalData.bitcoinPercentageOfMarketCap.toFloat())}%\n")
                    } else {
                        Text("Загрузка...")
                    }
                }
            }
        }
    }
}